@@ -1,9 +1,11 @@
-#!/usr/bin/env python
 # -*- coding=utf-8 -*-
-from __future__ import print_function, absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
+
+import sys
+
 import click
 import crayons
-import sys
+
 from . import __version__
 from .pythonfinder import Finder
 
@@ -11,16 +13,22 @@ from .pythonfinder import Finder
 @click.command()
 @click.option("--find", default=False, nargs=1, help="Find a specific python version.")
 @click.option("--which", default=False, nargs=1, help="Run the which command.")
-@click.option(
-    "--findall", is_flag=True, default=False, help="Find all python versions."
-)
+@click.option("--findall", is_flag=True, default=False, help="Find all python versions.")
 @click.option(
     "--version", is_flag=True, default=False, help="Display PythonFinder version."
 )
-@click.option("--ignore-unsupported/--no-unsupported", is_flag=True, default=True, envvar="PYTHONFINDER_IGNORE_UNSUPPORTED", help="Ignore unsupported python versions.")
-@click.version_option(prog_name='pyfinder', version=__version__)
+@click.option(
+    "--ignore-unsupported/--no-unsupported",
+    is_flag=True,
+    default=True,
+    envvar="PYTHONFINDER_IGNORE_UNSUPPORTED",
+    help="Ignore unsupported python versions.",
+)
+@click.version_option(prog_name="pyfinder", version=__version__)
 @click.pass_context
-def cli(ctx, find=False, which=False, findall=False, version=False, ignore_unsupported=True):
+def cli(
+    ctx, find=False, which=False, findall=False, version=False, ignore_unsupported=True
+):
     if version:
         click.echo(
             "{0} version {1}".format(
@@ -56,7 +64,20 @@ def cli(ctx, find=False, which=False, findall=False, version=False, ignore_unsup
         click.secho("Searching for python: {0!s}".format(find.strip()), fg="yellow")
         found = finder.find_python_version(find.strip())
         if found:
+            py = found.py_version
+            comes_from = getattr(py, "comes_from", None)
+            if comes_from is not None:
+                comes_from_path = getattr(comes_from, "path", found.path)
+            else:
+                comes_from_path = found.path
+            arch = getattr(py, "architecture", None)
             click.secho("Found python at the following locations:", fg="green")
+            click.secho(
+                "{py.name!s}: {py.version!s} ({py.architecture!s}) @ {comes_from!s}".format(
+                    py=py, comes_from=comes_from_path
+                ),
+                fg="yellow",
+            )
             sys.exit(0)
         else:
             click.secho("Failed to find matching executable...", fg="yellow")
